/* Template for a specific morphology code. */

#define PY_SSIZE_T_CLEAN
#include <numpy/arrayobject.h>
#include <Python.h>


#define OP(a, b)  (((a) < (b)) ? (a) : (b))  // min
// #define OP(a, b)  (((a) > (b)) ? (a) : (b))  // max


{morpho_patch}


{morpho_valid}


static PyObject* py_morpho(PyObject* Py_UNUSED(self), PyObject* args, PyObject* kwargs) {
  // declaration
  static char *kwlist[] = {"src", "dst", NULL};
  PyArrayObject *src, *dst = NULL;

  // parse and check
  if ( !PyArg_ParseTupleAndKeywords(
    args, kwargs, "O!|O!", kwlist,
    &PyArray_Type, &src, &PyArray_Type, &dst
    )
  ) {
    return NULL;
  }
  if ( PyArray_NDIM(src) != {dim} ) {
    PyErr_SetString(PyExc_ValueError, "'src' requires {dim} dimensions");
    return NULL;
  }

  // allocate the output array
  if ( dst == NULL ) {
    dst = (PyArrayObject *)PyArray_EMPTY({dim}, PyArray_SHAPE(src), PyArray_TYPE(src), 0);
    if ( dst == NULL ) {
        return PyErr_NoMemory();
      }
  } else {
    // verification of dst
    if ( PyArray_NDIM(dst) != {dim} ) {
      PyErr_SetString(PyExc_ValueError, "'dst' requires {dim} dimensions");
      return NULL;
    }
    for ( long int i = 0; i < {dim}; ++i) {
      if ( PyArray_DIM(src, i) != PyArray_DIM(dst, i) ) {
        PyErr_SetString(PyExc_ValueError, "'src' and 'dst' have not the same shapes");
        return NULL;
      }
    }
    if ( PyArray_TYPE(src) != PyArray_TYPE(dst) ) {
      PyErr_SetString(PyExc_ValueError, "'src' and 'dst' have not the same dtype");
      return NULL;
    }
  }

  // main code
  morpho_valid(dst, src);

  return (PyObject*)dst;
}


static PyMethodDef morphoMethods[] = {
  {
    "morpho", (PyCFunction)py_morpho, METH_VARARGS | METH_KEYWORDS,
    R"({func_title}

    {func_desc}
    )"
  },
  {NULL, NULL, 0, NULL}
};


static struct PyModuleDef morpho = {
  PyModuleDef_HEAD_INIT,
  "morpho",
  "This is an autogenerated c module.",
  -1,
  morphoMethods
};


PyMODINIT_FUNC PyInit_morpho(void)
{
  import_array();
  if ( PyErr_Occurred() ) {
    return NULL;
  }
  return PyModule_Create(&morpho);
}

